<?php

namespace App\Http\Controllers\Admin;
use Illuminate\Support\Facades\Schema;
use App\Enums\StatusEnum;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Settings;
use App\Rules\General\FileExtentionCheckRule;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\RedirectResponse;
use Illuminate\Contracts\View\View;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Artisan;

use Illuminate\Support\Facades\Log;

class SettingController extends Controller
{

    /**
     * get all system settings
     * @return View
     */
    public function index() :View
    {
        $title         = "General Setting";
        $timeLocations = timezone_identifiers_list();
        $countries     = json_decode(file_get_contents(resource_path('views/partials/country_file.json')));
        return view('admin.setting.index', compact('title', 'timeLocations','countries'));
    }



    /**
     * get ai settings
     * @return View
     */
    public function aiSettings() :View
    {
        if(auth_user()->agent != StatusEnum::false->status()){
            abort(403,unauthorized_message());
        }
        $title         = "Ai Configuration";
        return view('admin.setting.ai_config', compact('title'));
    }


    /**
     * view system config
     * @return View
     */
    public function systemConfiguration() :View
    {
        $title = "Sytem Configuration";
        return view('admin.setting.system_configuration', compact('title'));
    }



    /**
     * update pluging settings
     * @return RedirectResponse
     */
    public function pluginSetting(Request $request) :RedirectResponse{

        $this->updateSettings($request->site_settings);
        if(isset($request->site_settings['google_recaptcha'])){
           if($request->site_settings['google_recaptcha']['status'] == (StatusEnum::true)->status()){
              Settings::where('key','default_recaptcha')->update(
                [
                    'value' => (StatusEnum::false)->status()
                ]
                );
           }
        };
        optimize_clear();
        return back()->with('success',translate('Plugin Setting has been updated'));

    }

    /**
     * update ticket settings
     * @return RedirectResponse
     */
    public function ticketSetting(Request $request) :RedirectResponse{

        $request->validate([
            'ticket_setting.*.labels'               => 'required',
            'ticket_setting.*.type'                 => 'required',
            'ticket_setting.*.required'             => 'required',
            'ticket_setting.*.placeholder'          => 'required',
            'ticket_setting.*.default'              => 'required',
            'ticket_setting.*.multiple'             => 'required',

        ],
        [
            'ticket_setting.*.labels.required'      => translate('All Labels Field Is Required'),
            'ticket_setting.*.type.required'        => translate('All Type Field Is Required'),
            'ticket_setting.*.required.required'    => translate('All Required Field Is Required'),
            'ticket_setting.*.placeholder.required' => translate('All Placeholder Field Is Required'),
            'ticket_setting.*.default.required'     => translate('All Default Field Is Required'),
            'ticket_setting.*.multiple.required'    => translate('All Multiple Field Is Required'),
        ]);

        $newTicketSetting = [];

        foreach ($request->ticket_setting as $index => $field) {
            $newField = $field;
            if (is_null($field['name'])) {
                $newField['name'] = strtolower(str_replace(" ","_",$newField['labels']));
            }
            $newTicketSetting[$index] = $newField;
        }
        $request->merge(['ticket_setting' => $newTicketSetting]);

        try {
            Settings::where('key','ticket_settings')->update([
                'value' => json_encode($newTicketSetting)
            ]);

            $this->updateSettings($request->site_settings);

          } catch (\Throwable $th) {
             return back()->with('error',translate('Something Went wrong!! Fail to update'));
          }
          optimize_clear();
          return back()->with('success',translate('Ticket Setting has been updated'));

    }

    /**
     * update logo settings
     * @return RedirectResponse
     */
    public function logoSetting(Request $request) :RedirectResponse{

        $request->validate([
            'site_settings.site_logo_lg'   =>  ['image', new FileExtentionCheckRule(json_decode(site_settings('mime_types'),true))],
            'site_settings.site_logo_sm'   =>  [ 'image' ,new FileExtentionCheckRule(json_decode(site_settings('mime_types'),true))],
            'site_settings.site_favicon'   =>  [ 'image', new FileExtentionCheckRule(json_decode(site_settings('mime_types'),true))],
            'site_settings.frontend_logo'  =>  [ 'image', new FileExtentionCheckRule(json_decode(site_settings('mime_types'),true))]
        ],
        );

        $logo_section =  [];

        if ( isset($request['site_settings']['site_logo_lg']) && is_file($request['site_settings']['site_logo_lg']->getPathname())) {
            $site_logo = site_settings('site_logo_lg');
            try {
                $logo_section ['site_logo_lg'] =   storeImage($request['site_settings']['site_logo_lg'], getFilePaths()['site_logo']['path'], null,   $site_logo);
            } catch (\Throwable $th) {
                return back()->with('error',translate('Something Went wrong!! Check Your Directory Permissions'));
            }
        }

        if ( isset($request['site_settings']['site_logo_sm']) && is_file($request['site_settings']['site_logo_sm']->getPathname())) {
            $site_logo = site_settings('site_logo_sm');
            try {
                $logo_section ['site_logo_sm'] =   storeImage($request['site_settings']['site_logo_sm'], getFilePaths()['site_logo']['path'], null,   $site_logo);
            } catch (\Throwable $th) {
                return back()->with('error',translate('Something Went wrong!! Check Your Directory Permissions'));

            }

        }

        if ( isset($request['site_settings']['site_favicon']) && is_file($request['site_settings']['site_favicon']->getPathname())) {
            $site_logo = site_settings('site_favicon');
            try {
                $logo_section ['site_favicon'] =   storeImage($request['site_settings']['site_favicon'], getFilePaths()['site_logo']['path'], null,   $site_logo);
            } catch (\Throwable $th) {
                return back()->with('error',translate('Something Went wrong!! Check Your Directory Permissions'));

            }
        }

        if ( isset($request['site_settings']['frontend_logo']) && is_file($request['site_settings']['frontend_logo']->getPathname())) {
            $site_logo = site_settings('frontend_logo');
            try {
                $logo_section ['frontend_logo']  =   storeImage($request['site_settings']['frontend_logo'], getFilePaths()['site_logo']['path'], null,   $site_logo);
            } catch (\Throwable $th) {
                return back()->with('error',translate('Something Went wrong!! Check Your Directory Permissions'));

            }
        }

        $this->updateSettings($logo_section);

        optimize_clear();
        return back()->with('success',translate('logo  has been updated'));
    }

    /**
     * update  settings
     * @return RedirectResponse
     */
    public function store(Request $request) :RedirectResponse
    {
        $validations = $this->settings_validations($request->site_settings);
        $request->validate( $validations['rules'],$validations['message']);
        $this->updateSettings($request->site_settings);
        optimize_clear();
        return back()->with('success',translate('General Setting has been updated'));
    }

    /**
     * update  settings
     * @param array $request_data
     * @return RedirectResponse
     */
    public function updateSettings(array $request_data) :void{

        foreach(($request_data) as $key=>$value){

            if($key == 'aws_s3' || $key == 'pusher_settings' || $key == 'social_login' || $key == 'google_recaptcha' ){
                $value = json_encode($value);
            }

           try {

                Settings::updateOrInsert(
                    ['key'    => $key],
                    ['value'  => $value]
                );
           } catch (\Throwable $th) {

           }
        }
        optimize_clear();
    }


    /**
     * clear cache
     * @return RedirectResponse
     */
    public function cacheClear() :RedirectResponse
    {
        optimize_clear();
        return back()->with('success',translate('Cache Cleared Successfully'));
    }

    /**
     * clear cache
     * @return View
     */
    public function systemInfo() :View
    {
        $title = "System Information";
        $systemInfo = [
            'laravel_version' => app()->version(),
            'server_detail'   => $_SERVER,
            'php_version'     => phpversion(),
        ];
        return view('admin.system_info',compact('title','systemInfo'));
    }

    /**
     * settings validations
     * @return array
     */
    public function settings_validations(array $request_data ,string $key = 'site_settings') :array{

        $rules = [];
        $message = [];
        foreach(array_keys($request_data) as $data){

            $rules[$key.".".$data] ='required';
            $message[$key.".".$data.'.required'] = ucfirst(str_replace('_',' ',$data)).' '.translate('Feild is Required');
        }

        return [
            'rules'    =>  $rules,
            'message'  => $message
        ];
    }


    /**
     * update setting status
     * @return JsonResponse
     */
    public function settingsStatusUpdate(Request $request) : JsonResponse{


        Settings::updateOrInsert(
            ['key'   => $request->data['key'] ],
            ['value' =>  $request->data['status']]
        );

        if($request->data['key'] == 'app_debug'){
            
            $response['reload']  = true;
            if($request->data['status'] ==  (StatusEnum::true)->status()){
                update_env('APP_DEBUG=false',"APP_DEBUG=true");
            }
            else{
                update_env('APP_DEBUG=true',"APP_DEBUG=false");
            }
        }

        if($request->data['key'] == 'default_recaptcha' &&  $request->data['status'] ==  (StatusEnum::true)->status()){

            $google_recaptcha           =  json_decode(site_settings('google_recaptcha'),true);
            $google_recaptcha['status'] =  (StatusEnum::false)->status();
            Settings::where('key','google_recaptcha')->update([
                'value' => json_encode($google_recaptcha )
            ]);
        }


        if (
            ($request->data['key'] == 'auto_ticket_assignment' || $request->data['key'] == 'group_base_ticket_assign')
            && $request->data['status'] == (StatusEnum::true)->status()
        ) {
            $updateKey = ($request->data['key'] == 'auto_ticket_assignment') ? 'group_base_ticket_assign' : 'auto_ticket_assignment';
            $response['reload']  = true;
            Settings::where('key', $updateKey)->update([
                'value' => (StatusEnum::false)->status()
            ]);
        }



        optimize_clear();
        $response['status']  = true;

        $response['message'] = translate('Status Updated Successfully');
        return response()->json($response);
    }



    /** system update  */

     public function sysyemUpdate() :RedirectResponse{

        $status = 'error';
        try {
            $status     = 'success';
            $message    = 'Your system is currently running the latest version.';

            $newVersion = (double) Arr::get(config("requirements")['core'],'appVersion',1.0);
     
            if(check_for_update()){

                $migrations = [
                    "database/migrations/2023_12_17_104245_add_column_to_priorities.php",
                    "database/migrations/2023_12_17_121249_add_column_to_admins.php",
                    "database/migrations/2023_12_17_121756_add_column_to_support_messages.php",
                    "database/migrations/2023_12_18_104510_create_visitors_table.php"
                ];
                
                Artisan::call('db:seed',[
                    '--force' => true
                ]);

                foreach($migrations as $migration){
                    Artisan::call('migrate', [
                        '--force' => true,
                        '--path' => $migration,
                    ]);
                }
    
                Log::info('Database seeder and migration executed');
                Settings::updateOrInsert(
                    ['key'    => "app_version"],
                    ['value'  => $newVersion]
                );
                optimize_clear();
                $message    = 'System updated successfully';
            }
        } catch (\Exception $ex) {
            $message = \strip_tags( $ex->getMessage());
        }

        return back()->with($status,translate($message));
    }



}
