<?php

namespace App\Http\Controllers;

use App\Enums\StatusEnum;
use App\Http\Controllers\Controller;
use App\Models\Admin;
use App\Models\Language;
use Carbon\Carbon;
use Gregwar\Captcha\CaptchaBuilder;
use Illuminate\Support\Facades\Session;
use Gregwar\Captcha\PhraseBuilder;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;
use Illuminate\View\View;
use Closure;

class CoreController extends Controller
{
    
      /**chnage language method */
      public function languageChange(string $code ){
        $response['status']  = "success";
        $response['message'] = translate('Language Switched Successfully');

        if(!Language::where('code', $code)->exists()){
            $code = 'en';
        }
        optimize_clear();
        session()->put('locale', $code);
        app()->setLocale($code);
        return back()->with("success",translate('Language Switched Successfully'));
      }

      public function defaultImageCreate(string $size = null){

          $width            = explode('x',$size)[0];
          $height           = explode('x',$size)[1];
          $image            = imagecreate($width, $height);
          $fontFile         = realpath('assets/font') . DIRECTORY_SEPARATOR . 'RobotoMono-Regular.ttf';
          if($width > 100 && $height > 100){
              $fontSize     = 30;
          }else{
              $fontSize     = 5;
          }
          $text = $width . 'X' . $height;
          $backgroundcolor  = imagecolorallocate($image, 237, 241, 250);
          $textcolor        = imagecolorallocate($image, 107, 111, 130);
          imagefill($image, 0, 0, $textcolor);
          $textsize         = imagettfbbox($fontSize, 0, $fontFile, $text);
          $textWidth        = abs($textsize[4] - $textsize[0]);
          $textHeight       = abs($textsize[5] - $textsize[1]);
          $xx = ($width - $textWidth) / 2;
          $yy = ($height + $textHeight) / 2;
          header('Content-Type: image/jpeg');
          imagettftext($image, $fontSize, 0, $xx, $yy, $backgroundcolor , $fontFile, $text);
          imagejpeg($image);
          imagedestroy($image);
      }


      
     /**
     * genarate default cpatcha code
     *
     * @return void
     */
    public function defaultCaptcha(int | string $randCode){
        
        $phrase   = new PhraseBuilder;
        $code     = $phrase->build(4);
        $builder  = new CaptchaBuilder($code, $phrase);
        $builder->setBackgroundColor(220, 210, 230);
        $builder->setMaxAngle(25);
        $builder->setMaxBehindLines(0);
        $builder->setMaxFrontLines(0);
        $builder->build($width = 100, $height = 40, $font = null);
        $phrase   = $builder->getPhrase();

        if(Session::has('gcaptcha_code')) {
            Session::forget('gcaptcha_code');
        }
        Session::put('gcaptcha_code', $phrase);
        header("Cache-Control: no-cache, must-revalidate");
        header("Content-Type:image/jpeg");
        $builder->output();
    }


    public function cron(){

        if(site_settings('auto_best_agent') == StatusEnum::true->status()){

            $agents = Admin::with(["response","tickets"])
            ->withCount(["response","tickets"])
            ->agent()
            ->active()
            ->get();

            foreach($agents->chunk(paginateNumber()) as $chunksAgents){

                foreach($chunksAgents as $agent){
   

                    if($agent->response_count > 0){

                        $reponseTimeReq  = (int) site_settings('avg_response_time');
                        $avgResponseTime = (int) round($agent->response->avg("response_time")*60);

                        if($agent->response_count >= site_settings('number_of_tickets')  && $avgResponseTime <=  $reponseTimeReq){
                            
                            $agent->best_agent = StatusEnum::true->status();
                            $agent->save();

                        }

                    }
             
                }
            }
        }


        session()->put('last_cron_run',Carbon::now());
    }



    public function aiContent(Request $request) : array{

   

        if(site_settings("open_ai")  == StatusEnum::false->status()){
            return [
                "status"  => false,
                "message" => translate("Ai Module is Currently of Now"),
            ];
        }

        $request->validate([
            'ai_creativity_level'=>['required',Rule::in(['1','0','0.5'])],
            'number_of_result'=>"required|int",
            'max_result_length'=>"required|int",
            'question'=>"required",
        ]);

        $text = '';


        $message = translate("Set Up Your OpenAi API key Then try Again") ;
        $apiKey  =  site_settings('open_ai_key');

        if(!$apiKey){
            return ([
                'status'  => false,
                'message' => $message 
            ]);
        }

        $result  = $this->aiResult($request);

        $counter = 1;
        foreach ($result['choices'] as $value)
        {
            $text .= $counter . '. ' . ltrim($value['text']) . "\r\n\r\n\r\n";
            $counter++;
        }
        

        return ([
            'status'  => true,
            'message' => $text
        ]);

    }
    


        /**
         * generate ai reply for ticket
         * 
         */
      public  function aiResult($request){

        $apiKey      =  site_settings('open_ai_key');

        $client      = \OpenAI::client($apiKey);

        $max_results = (int) $request->number_of_result;
        $max_tokens  = (int) $request->max_result_length;
      
        return  $client->completions()->create([

            'model'        => 'text-davinci-003',
            "temperature"  => (float)$request->ai_creativity_level,
            'prompt'       => $request->question,
            'max_tokens'   => $max_tokens,
            'n'            => $max_results,

        ]);

     }



     public  function maintenanceMode() :View | RedirectResponse{

        if(site_settings('maintenance_mode') == (StatusEnum::false)->status() ){
            return redirect()->route('home');
        }
        $title = 'Maintenance Mode';
        return view('maintenance_mode' ,compact('title'));

     }



     
    public function security() :View{

        if(site_settings('dos_prevent') == StatusEnum::true->status() && !session()->has('dos_captcha')){

            return view('dos_security',[
                "title"    =>  translate('Too many request')
            ]);
        }
        abort(403);
    }


    public function securityVerify(Request $request) :RedirectResponse{

    
        $request->validate([
            "captcha" =>   ['required' , function (string $attribute, mixed $value, Closure $fail) {
                if (strtolower($value) != strtolower(session()->get('gcaptcha_code'))) {
                    $fail(translate("Invalid captcha code"));
                }
            }]
        ]);

        session()->forget('gcaptcha_code');
        session()->forget('security_captcha');
        session()->put('dos_captcha',$request->input("captcha"));

        $route = 'home';
        if(session()->has('requested_route')){
            $route = session()->get('requested_route');
        }

        return redirect()->route($route);
    }

    
  
}
