<?php

namespace App\Http\Controllers;

use App\Http\Controllers\Controller;
use App\Models\Article;
use App\Models\ArticleCategory;
use App\Models\Category;
use App\Models\Contact;
use App\Models\Page;
use App\Models\Subscriber;
use Illuminate\Http\Request;
use Illuminate\Contracts\View\View;
use Illuminate\Http\RedirectResponse;
class FrontendController extends Controller
{

    public function index() :View{
  
        $title      = "Home";
        $categories = Category::with(['faq'])->active()->get();
        return view('frontend.home' ,compact('title','categories'));
    }

    public function articles(Request $request,$slug = null ,$id = null) :View{
        
        $item       = Category::where('id',$id)->first();
        $categories = ArticleCategory::active()
        ->withCount(['articles'])
        ->whereHas("articles" , function($q) use($item){
            return $q->with(['items'])
                ->active()
                ->where('category_id',$item->id)
                ->orderBy("serial_id",'ASC');})
        ->with(['articles'=> function($q) use($item){
            return $q->with(['items'])->active()->where('category_id',$item->id)->orderBy("serial_id",'ASC');
        }])->orderBy("serial_id",'ASC')->get();

        $articleDetails = $categories->first()?->articles->first();
 
        $title          = ucfirst(str_replace("_"," ",$slug));

        return view('frontend.articles' ,compact('title','categories','articleDetails','item'));
    }

    public function articleView($item,$category,$title,$id) :View{

       $articleDetails  = Article::where("id",$id)->first();
       $item            = Category::where('id',$articleDetails->category_id)->first();

       $title           = str_replace("_"," ",$articleDetails->name);

       $categories      = ArticleCategory::active()
       ->whereHas("articles" , function($q) use($item){
        return $q->with(['items'])
            ->active()
            ->where('category_id',$item->id)
            ->orderBy("serial_id",'ASC');})
       ->with(['articles'=> function($q) use($item){
        return $q->with(['items'])
            ->active()
            ->where('category_id',$item->id)
            ->orderBy("serial_id",'ASC');
        }])->orderBy("serial_id",'ASC')->get();


       return view('frontend.articles' ,compact('title','categories','articleDetails','item'));
    }


    public function articleSearch(Request $request) : view | string  {


        $request->validate([
            "search"          => 'required'
        ],
        [
            "search.required" => translate('Type Something Then Try Again')
        ]);

        $title                = "Article Search";

        $articles             = Article::with(['items','category'])
        ->where('description',"like", "%".$request->search."%")
        ->orWhere('name',"like", "%".$request->search."%")
        ->get();

        if(request()->ajax()){

            return json_encode([
                
                "status"        => true ,
                "article_html"  => view('frontend.partials.articles', compact('articles'))->render(),
            ]);
        }

        return view('frontend.article_view' ,compact('title','articles'));
    }


    public function pages($slug) :View{

        $page  = Page::active()->where('slug',$slug)->first();
        $title = $page->title;
        return view('frontend.pages' ,compact('title','page'));
    }

    public function subscribe(Request $request) :RedirectResponse{

        $request->validate([
            'email'           => 'required|unique:subscribers,email',
        ],[
            'email.required'  => translate('Email Is Required'),
            'email.unique'    => translate('Already Subscribed'),
        ]);

        Subscriber::create([
            'email'           =>$request->email
        ]);
         
        return back()->with('success',translate('Subscribed Successfully'));
    }



    public function contactStore(Request $request) :RedirectResponse{

        $request->validate([
            'email'            =>  'required',
            'name'             =>  'required',
            'subject'          =>  'required',
            'message'          => 'required',
    
        ],[
            'email.required'   => translate('Email Is Required'),
            'name.required'    => translate('Name Is Required'),
            'subject.required' => translate('Subject is Required'),
            'message.required' => translate('Message is Required'),
        ]);

        Contact::create([
            'name'             => $request->name,
            'email'            => $request->email,
            'subject'          => $request->subject,
            'message'          => $request->message,
        ]);
 
        return back()->with('success',translate('Successfully Contacted'));
    }


}
