<?php

namespace App\Http\Controllers;

use App\Enums\StatusEnum;
use App\Enums\TicketStatus;
use App\Http\Controllers\Controller;
use App\Http\Requests\TicketStoreRequest;
use App\Http\Services\TicketService;
use App\Models\Category;
use App\Models\Priority;
use App\Models\SupportTicket;
use Carbon\Carbon;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Contracts\View\View;
class GuestTicketController extends Controller
{

    public $ticketService;
    public function __construct(TicketService $ticketService)
    {
      $this->ticketService = $ticketService;
    }

    /**
     * create a new ticket
     *
     * @return View
     */
    public function create() :View{

       $title         = "Create Ticket";
       $categories    = Category::where('status',(StatusEnum::true)->
       status())
       ->active()
       ->where("ticket_display_flag",StatusEnum::true->status())
       ->get();
       $priorites     = Priority::active()->get();
       $ticket_fields = json_decode(site_settings('ticket_settings'),true);
       return view('ticket',compact('ticket_fields','categories','title','priorites'));
    }


    /**
     * Create A new Ticket
     *
     * @param TicketStoreRequest $request
     * @return \Illuminate\Http\RedirectResponse
     */
     public function store(TicketStoreRequest $request) :\Illuminate\Http\RedirectResponse {

        $message    = null;

        if(isset($request->ticket_data["attachments"][0])){
          $message  =   ticket_file_validation($request->ticket_data["attachments"]);
        }

        if($message){
            return  back()->with('error',$message);
        }

        $user = auth_user('web');
        if($user) {

            $ticketData = $request->input('ticket_data', []);
            $ticketData['name']   = $user->name;
            $ticketData['email']  = $user->email;

            $request->merge([
                "ticket_data" => $ticketData,
            ]);
        }
        $ticket = $this->ticketService->storeTicket($request->except('_token')['ticket_data']);

        return redirect()->back()->with('ticket_created',
           '<h5 class="mb-2">Ticket Successfully Created</h5>'.
          "</b><p class='text-black'> Notification: Please Review Your Email</p>".
          "<p class='text-black'>
              Ticket Issued: Your TicketId Is. <b class='text-primary'>".$ticket->ticket_number ."</b>
          </p>".

          '
          <div class="mt-3">

          <a class="Btn secondary-btn btn--sm w-50 btn-icon-hover d-flex align-items-center justify-content-center" href="' . route('ticket.reply', $ticket->ticket_number).'"><span>' . ('Your Ticket') . '</span><i class="bi bi-arrow-right-short fs-20"></i>  </a>
           </div>' );

     }


     /**
      * Reply to specific ticket
      *
      * @param string $ticketId
      * @return View
      */
      public function reply(string $ticketId) :View{

        $title     = 'Reply Ticket';
        $ticket    = SupportTicket::with(['messages'])->where('ticket_number',$ticketId)->firstOrFail();
        $messages  = $ticket->messages->whereNotNull('admin_id')->where('seen', StatusEnum::false->status());
        foreach ($messages as $message) {
           $message->update(['seen' => StatusEnum::true->status()]);
        }

        return view('reply',compact('ticket','title'));

      }


      /**
       * Undocumented function
       *
       * @param Request $request
       * @return mixed
       */
      public function search(Request $request) :mixed {

        $view    = "search_ticket";
        $title   = 'Search Ticket';

        if($request && $request->isMethod('post')){

            $request->validate([
              'ticket_number'          => "required",
              'email'                  => "required",
            ],[
              "ticket_number.required" =>translate('Please Enter Your Ticket Number'),
              "email.required"         =>translate('Please Enter Your Email'),
            ]);

            $title  = 'Reply Ticket';
            $ticket = SupportTicket::with(['messages'])
            ->where('ticket_number',$request->ticket_number)
            ->where('email',$request->email)
            ->first();

            if(!$ticket){
              return back()->with('error',translate('Ticket Not Found'));
            }

            $messages = $ticket->messages->whereNotNull('admin_id')->where('seen', StatusEnum::false->status());

            foreach ($messages as $message) {
                $message->update(['seen' => StatusEnum::true->status()]);
            }

            return \redirect()->route("ticket.reply",$ticket->ticket_number);
        }

        return view("search_ticket" ,compact('title'));
      }



    public function solveRequest(string | int $ticketId , bool  $status) : RedirectResponse {


      $ticket = SupportTicket::where('id', $ticketId)->firstOrfail();

      if($status){

          $ticket->solved_request  = SupportTicket::ACCEPTED;
          $ticket->solved_at       = Carbon::now();
      }
      else{

        $ticket->solved_request  = SupportTicket::REJECTED;
        $ticket->status          = TicketStatus::PROCESSING->value;


      }

      $ticket->save();

      return back()->with("success",translate("Thanks!! For Your Response"));

    }


}
