<?php

namespace App\Http\Controllers\User;

use App\Http\Controllers\Controller;
use App\Enums\StatusEnum;
use App\Enums\TicketStatus;
use App\Http\Services\TicketService;
use App\Models\Category;
use App\Models\SupportMessage;
use App\Models\SupportTicket;
use App\Rules\General\FileExtentionCheckRule;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\View\View;

use Illuminate\Support\Facades\DB;
class UserTicketController extends Controller
{
    


    public $ticketService;

    public function __construct(TicketService $ticketService)
    {
        $this->ticketService = $ticketService;

    }

    /**
     * get ticket list
     */
     public function index(Request $request):Mixed{

        $title     = 'Ticket List';
        $tickets   = SupportTicket::with(['linkedPriority'])->where('user_id',auth_user('web')->id)
        ->withCount(['messages' => function($q){
            $q->whereNotNull('admin_id')
            ->where('seen',StatusEnum::false->status());
        }])->latest();

        if($request->ajax()){

           return ($this->processData($tickets,$request));
        
        }
        return view('user.ticket.index',compact('title'));
     }


     /**
      * mark  method
      */
      public function mark(Request $request){
   
        $request->validate([
            'ticket_id'            => "required|array|exists:support_tickets,id"
        ],[
            "ticket_id.required"   => translate('Ticket Id Feild Is Required'),
            "ticket_id.array"      => translate('Ticket Id Feild Must Be An Array'),
            "ticket_id.exists"     => translate('Invalid Tickets Selected'),
        ]);

        if($request->status){

            $message = translate('Status Updated Successfully');
            SupportTicket::whereIn('id',$request->ticket_id)->update([

                'status'=> $request->status
             ]);
        }
        elseif($request->assign){
            $this->assignTicket($request);
            $message = translate('Ticket Assigned Successfully');
        }
        else{
            $this->deleteTicket($request->ticket_id);
            $message = translate('Ticket Deleted Successfully');
        }
        return back()->with('success', $message );

      }


      /**
       * delete tickets 
       */
      public function deleteTicket($ticketIds){

        foreach($ticketIds as $ticket_id){

            $ticket = SupportTicket::with(["messages",'agents'])
            ->withCount('messages')
            ->where('status',TicketStatus::CLOSED->value)
            ->where('id',$ticket_id)
            ->first();
            if($ticket){
                if($ticket->messages_count > 0){
                    foreach($ticket->messages as $message){

                        $ticketFiles = json_decode($message->file,true);
                        $this->unlinkFile($ticketFiles ? $ticketFiles :[]  , getFilePaths()['ticket']['path']); 
               
                        $message->delete();
                    }
                }
                $ticket->agents()->detach();
                $ticket->delete();
            }
        }
      }



    /** unlink ticket files */
    public function unlinkFile($files ,$location){
        foreach($files as $file){
            remove_file($location ,$file );
        }
    }


     /** proceess ticket data */
     public function processData($tickets,$request) :string{



        try {

            $ticketData       = $tickets;
            $status           = true;
            $ticket_status    = [];
    
            $categories       = Category::withCount(['tickets as tickets_count' => function($q){
                return $q->where('user_id',auth_user('web')->id);
            }])
            ->having("tickets_count" ,">", 0)
            ->where("ticket_display_flag",StatusEnum::true->status())
            ->latest()
            ->get();
        
            $category_active  = $request->category_id ? $request->category_id : null;
            $status_active    = $request->status ? $request->status : null;
            $tickets          =  $tickets->filter($request)
            ->paginate(paginateNumber(), ['*'], 'page', $request->page) ?? [] ;
    
            $nextPageNumber   = $tickets->currentPage() + 1;
            $status = count($tickets)  == 0 ? false :true;
    
            $currentPage      = $tickets->perpage() * $tickets->currentpage() > $tickets->total()  ? $tickets->total()  :  $tickets->perpage() * $tickets->currentpage();
    
            $ticketStatus     = TicketStatus::toArray();
            
            $tickets_counter  =  SupportTicket::where('user_id',auth_user('web')->id)
            ->filter($request,false)
            ->groupBy('status')
            ->select('status', DB::raw('count(*) as count'))
            ->pluck('count', 'status')
            ->toArray();
    
    
            return json_encode([
                'total_ticket'        => $tickets->total(),
                'current_page_ticket' => $currentPage,
                'status'              => $status,
                "ticket_html"         => view('user.ticket.list', compact('tickets'))->render(),
                "categories_html"     => view('admin.ticket.categories', compact('categories','tickets','category_active'))->render(),
                "status_html"         => view('admin.ticket.status', compact('ticketStatus',"tickets_counter",'status_active'))->render(),
            ]);
        } catch (\Throwable $ex) {
           
        }

       
     }



     /**
      * view ticket details
      */
      public  function view(string $id) :View | RedirectResponse
      {

        $title = 'View Ticket';
        $ticket = SupportTicket::with(['agents','category' ,'messages','linkedPriority'])
        ->where('user_id',auth_user('web')->id)
        ->where('ticket_number',$id)->first();



        if(!$ticket){
            return back()->with('error',translate('Ticket Not Found'));
        }
       
        $messages = $ticket->messages->whereNotNull('admin_id')->where('seen', StatusEnum::false->status());

        
        foreach ($messages as $message) {
            $message->update(['seen' => StatusEnum::true->status()]);
        }


        return view('user.ticket.details',compact('title','ticket'));
      }

      /**
       * reply to a ticket 
       */
       public  function reply(Request $request) :RedirectResponse
       {
            $request->validate([
                'message'            => 'required',
                'ticket_id'          => 'required | exists:support_tickets,id',
                'files'              => [ new FileExtentionCheckRule(json_decode(site_settings('mime_types'),true),'Ticket File')]
            ],[
                "message.required"   => translate('Message Feild is Required'),
                "ticket_id.required" => translate('Ticket Id Feild Is Required'),
                "ticket_id.exists"   => translate('Invalid Tickets Selected'),
             
            ]);

            $ticket = SupportTicket::with(['agents'])->where('id',$request->ticket_id)->first();


            $ticket_files = [];

    
            if($request->has('files')){
                try {
                    $ticket_files = $this->ticketService->storeFiles( $request['files']);
                } catch (\Exception $ex) {

                    return back()->with('error', strip_tags($ex->getMessage()));
                }
            }
           
         

            $supportMessage                    = new SupportMessage();
            $supportMessage->support_ticket_id = $request->ticket_id;
            $supportMessage->admin_id          = null;

            $supportMessage->message           = $request->message;

            $supportMessage->file              = json_encode($ticket_files);
            $supportMessage->save();
            $status  = 'success';
            $message = translate("Replied Successfully");
            
            $route   = route('admin.ticket.view',$ticket->ticket_number);
            $notifyMessage = translate('Hello Dear !!!'). $ticket->name . " Just Replied To a Conversations";

            $mailCode = [
                "role"          => 'User',
                "name"          => $ticket->name ,
                "ticket_number" => $ticket->ticket_number,
                "link"          => $route
            ];

            $this->ticketService->notifyAgent($ticket,null, $notifyMessage ,"user_reply_admin" ,$route ,"TICKET_REPLY",
            $mailCode);


            $getAgent = $ticket->agents->first();
            
            if($getAgent && $getAgent?->agent == StatusEnum::true->status()){
                $this->ticketService->notifyAgent($ticket,$ticket->agents->first(), $notifyMessage ,"user_reply_agent" ,$route ,"TICKET_REPLY",
                $mailCode);
            }


            return back()->with($status,$message);

       }




       /**
        * download a file 
        */
        public function download(Request $request) {

            $request->validate([
                'name'  => 'required',
            ]);
            $url        =  download_file(getFilePaths()['ticket']['path'],$request->name);
            if(!$url){
                return back()->with('error',translate('File Not Found'));
            }
            return $url;

        }



   
        /**
         * export tickets 
         */
        public function export($extension) {

            $ticketData   =  SupportTicket::where('user_id',auth_user('web')->id)->latest();
            $view         = null;
            if($extension == 'pdf'){
                $view     = "export_pdf";
            }
            return $this->ticketService->exportData($extension,$ticketData,$view );
        }


        /**
         * delete a specific ticket
         */
        public function delete(int | string $id) :RedirectResponse{
            
            $this->deleteTicket((array)$id);
            $status  = 'success';
            $message = translate("Deleted Successfully");
            return redirect()->route('user.ticket.list')->with($status,$message);
        }


      

}
