<?php

namespace App\Http\Services;

use App\Enums\NotifyStatus;
use App\Enums\PriorityStatus;
use App\Enums\StatusEnum;
use App\Enums\TicketStatus;
use App\Http\Controllers\Controller;
use App\Http\Utility\SendNotification;
use App\Jobs\SendEmailJob;
use App\Jobs\SendSmsJob;
use App\Models\Admin;
use App\Models\AgentTicket;
use App\Models\Category;
use App\Models\Group;
use App\Models\Priority;
use App\Models\SupportMessage;
use App\Models\SupportTicket;
use Carbon\Carbon;
use Illuminate\Support\Arr;
use Pusher\Pusher;
use Pdf;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\DB;



class TicketService extends Controller
{


    public function storeTicket($request_data)
    {

      
        $defaultPriority         = Priority::default()->first();
        
        $ticket_files            = [];
        $ticket                  =  new SupportTicket();
        $ticket->ticket_number   =  generateTicketNumber();
        $ticket->user_id         =  auth_user('web')? auth_user('web')->id : null ;
        $ticket->category_id     =  $request_data['category'] ;
        $ticket->subject         =  $request_data['subject'] ;
        $ticket->name            =  $request_data['name'] ;
        $ticket->email           =  $request_data['email'] ;
        $ticket->status          =  TicketStatus::PENDING;
        $ticket->priority_id     =  Arr::get($request_data,'priority',@$defaultPriority?->id);
        $ticket->priority        =  PriorityStatus::MEDIUM;
        $ticket->ticket_data     =  json_encode(Arr::except($request_data, ['category','priority','name','email','subject','description','attachments']));
        $ticket->message         =  $request_data['description'];
        $ticket->save();

        $ticket_files            = [];

         try {
            if(isset($request_data['attachments'][0])){
                $ticket_files    =  $this->storeFiles( $request_data['attachments']);
            }
         } catch (\Throwable $th) {
            
         }


        $editor_files            =  [];
        $supportMessage          =  $this->ticketMessage($ticket, $ticket_files , $editor_files );
        $response                =  $this->assignTickets($ticket,$supportMessage);

        $message                 =  translate("You Have a New Unassigned Ticket");
        if($response){
            $message             =  translate("You Have a New Assigned Ticket");
        }

        $this->notifyAgent($ticket,null,$message,'new_ticket' ,route("admin.ticket.view",$ticket->ticket_number));
        
        $code = [
            'ticket_number'     => $ticket->ticket_number ,
            'link'              => route('ticket.reply',  $ticket->ticket_number)
        ];
        SendEmailJob::dispatch($ticket, "SUPPORT_TICKET_REPLY" ,$code);
        return $ticket;

    }

    /**
     * assgin tickets to agents
     */
     public function assignTickets($ticket,$supportMessage){


        $agents = Admin::with(['tickets'])
                ->agent()
                ->active()
                ->whereJsonContains('categories', (string)$ticket->category_id)
                ->get();

        $status = false;

        $groupBaseTicketAssignEnabled = site_settings('group_base_ticket_assign') == (StatusEnum::true)->status();
        $autoTicketAssignmentEnabled  = site_settings('auto_ticket_assignment')   == (StatusEnum::true)->status();

        if ($groupBaseTicketAssignEnabled) {
            $group = Group::with("members")->where("priority_id", $ticket->priority_id)->first();
            $assignTo = $group ? $group->members : $agents;
            $this->manualTicketAssignment($assignTo, $ticket , "Ticket Assigned By Priority Group");
            $status = true;
        } elseif (count($agents) > 0) {
            
            if($autoTicketAssignmentEnabled){
                $response = $this->autoTicketAssignment($agents, $ticket);
                if (!$response) {
                    $this->manualTicketAssignment($agents, $ticket);
                }
            }
            else{
          
                $this->manualTicketAssignment($agents, $ticket);  
            }
     
            $status = true;
        } 

        return $status;
     }

 
      public function manualTicketAssignment($agents,$ticket ,$shortNotes = "Ticket Assigned Manually [BY CATEGORY]"){


          $bestAgents  = $agents->where('best_agent',StatusEnum::true->status());

          if($bestAgents->count() > 0){
            $agents    = $bestAgents ;
          }



          foreach($agents->chunk(paginateNumber()) as $chunkAgents){

            foreach($chunkAgents as $agent) {

                if($this->ticketPermissions($agent)){
                    
           
                    $data =   
                    [ 
                        "agent_id"    => $agent->id , 
                        "ticket_id"   => $ticket->id,
                        "short_notes" => $shortNotes
                    ];

                    $agent->tickets()->attach([$data]);
                    $this->notifyAgent($ticket,$agent,null,"new_ticket",route("admin.ticket.view",$ticket->ticket_number));
                   
                }
            }

         }
      }

      /**
       *  get ticket permissions
       */

       public function ticketPermissions($agent){
        
           $permissions =  json_decode($agent->permissions,true) ? json_decode($agent->permissions,true) :[] ;
            if(in_array("manage_tickets",$permissions)){
                
                return true;
            }
            return false;
       }


     /**
      * auto ticket assignments
      */
      public function autoTicketAssignment($agents,$ticket){

            $status = false;
            $lat    =  null;
            $lon    =  null;

             try {
                if($ticket->user){
                    $lat = $ticket->user->latitude;
                    $lon = $ticket->user->longitude;
                }
                else{
                    $address = get_address_by_ip(request()->ip());
                    if(isset($address['lon']) &&  isset($address['lat'])){
                        $lat = $address['lat'];
                        $lon = $address['lon'];
                    }
                }
                if($lat && $lon){
                    $agent      = $this->nearestAgent($lat,$lon,$agents);
                    
                    if($this->ticketPermissions($agent)){
                        $status = true;
                        $data   =   
                        [ 
                            "agent_id"    => $agent->id , 
                            "ticket_id"   => $ticket->id,
                            "short_notes" => "Ticket Assigned Automatically [BY lOCATION]" 
                        ];
                        $agent->tickets()->attach([$data]);
                        $this->notifyAgent($ticket,$agent,null,'new_ticket',route("admin.ticket.view",$ticket->ticket_number) );
                     }
                }
             } catch (\Throwable $th) {
                $status = false;
             }
             return $status ;
      }



      /**
       * notify agent
       */
       public function notifyAgent($ticket = null ,$agent = null,$message = null,$subKey ="new_ticket",$routeName = null, $templateCode = "SUPPORT_TICKET_REPLY",$mailCode = null){

        
            $routeName           = $routeName ?? route('admin.ticket.list');
            $message             =  $message ?? translate("Agent!! You Have Newly Assigned Ticket, From ") . $ticket->name ;
            $superAdmin          = Admin::where('agent',(StatusEnum::false)->status())->first();
            $agent               = $agent?? $superAdmin ;
            $notification        =  null;
            $mutedTicket         = (array) $agent->muted_ticket ;
            $agentNotifications  = $agent->notification_settings 
                                     ? json_decode($agent->notification_settings ,true) 
                                    :[];

    
            $mailCode            = $mailCode ?? [
                                                    'ticket_number' => @$ticket->ticket_number ,
                                                    'link'          =>  $routeName  
                                                ];
            
    
            $data = [
                'route'    => $routeName ,  
                'messsage' => $message
            ];


            if(site_settings("database_notifications") == (StatusEnum::true)->status()){

    
                $notify       = $agent->agent ==  StatusEnum::false->status() 
                                 ? NotifyStatus::SUPER_ADMIN 
                                 : NotifyStatus::AGENT;

                $notification = SendNotification::database_notifications($data,$agent,$superAdmin->id, $notify);


                $notifications = [
                    'counter'          => false,
                    'data'             => json_decode($notification->data,true),
                    'notification_for' => $notification->notification_for,
                    'notify_id'        => $notification->notify_id
                ];

                $pushEvent = false;

                if(isset($agentNotifications['browser'][$subKey]) 
                    && $agentNotifications['browser'][$subKey] == (StatusEnum::true)->status() ){
                    $notifications['for']             = $subKey;
                    $pushEvent                        = true;
                }

                if(!in_array($ticket->id , $mutedTicket )){
                    $pushEvent                        = true;
                    $notifications['play_audio']      = true;
                }

                if($pushEvent){     

                   $push_data ['notifications_data'] =  $notifications ;
                    try {
                        $this->triggerPusher($push_data);
                    } catch (\Exception $e) {
                        
                    }
                }

            }

            /** send mail notifications  to agent */
            if(isset($agentNotifications['email'][$subKey]) && $agentNotifications['email'][$subKey] == (StatusEnum::true)->status()){

                SendEmailJob::dispatch($agent, $templateCode ,$mailCode);
            }

            /** send sms notifications to agent */
            if(isset($agentNotifications['sms'][$subKey]) && $agentNotifications['sms'][$subKey] == (StatusEnum::true)->status()){
                SendSmsJob::dispatch($agent, $templateCode ,$mailCode);
            }

            if($agent->agent ==  StatusEnum::false->status() 
                && isset($agentNotifications['slack'][$subKey]) 
                && $agentNotifications['slack'][$subKey] == (StatusEnum::true)->status()){

                $data['ticket_id'] = $ticket->ticket_number;
                SendNotification::slack_notifications($agent, $data);
            }

           return $notification ;
    
       }


      public function nearestAgent($latitude,$longitude,$agents){

            return $agents->sortBy(function ($agent) use ($latitude,$longitude) {
                return calculate_distance($latitude, $longitude , $agent->latitude, $agent->longitude);
           })->first();
      }



    public function notifyAdmin($ticket){


        if(site_settings("database_notifications") == (StatusEnum::true)->status()){

            $superAdmin    = Admin::where('agent',(StatusEnum::false)->status())->first();

            $notify        = NotifyStatus::SUPER_ADMIN;

            $data = [
                'route'    => route('admin.ticket.view',$ticket->ticket_number) ,  
                'messsage' => auth_user()->name . " requested to mark this ticket as Solved"
            ];
          
            $notification  = SendNotification::database_notifications($data,$superAdmin,auth_user()->id , $notify);
        }

   
    }

    
    /**
     * store  ticket message
     */
     public function ticketMessage($ticket,$ticket_file,$editor_files ){

        $supportMessage                    = new SupportMessage();
        $supportMessage->support_ticket_id = $ticket->id;
        $supportMessage->message           = $ticket->message;
        $supportMessage->editor_files      = json_encode($editor_files);
        $supportMessage->file              = json_encode($ticket_file);
        $supportMessage->save();
        return $supportMessage;
     }

    /**
     * store ticket files
     */
     public function storeFiles($files ,$path = null):array
     {
        if(!$path){
            $path = getFilePaths()['ticket']['path'];
        }
        $storedFiles = [];
        foreach($files as $file){
            array_push($storedFiles,upload_new_file($file, getFilePaths()['ticket']['path']));
        }
        return $storedFiles ;
     }


     /**
      * config and trigger pusher 
      */
    public function triggerPusher($data){

        $pusher_settings =  json_decode(site_settings('pusher_settings'),true);

        $options = array(
            'cluster' => $pusher_settings['app_cluster'],
            'useTLS'  => true,
        );
        $pusher =  new Pusher(
            $pusher_settings['app_key'], $pusher_settings['app_secret'], $pusher_settings['app_id'], $options
        );

        $pusher->trigger($pusher_settings['chanel'], $pusher_settings['event'], $data);
    

    }

    /**
     * export ticket data
     */

     public function exportData($extension,$exportableData,$view = null){

        $headers = [
            'ticket_number',
            'name',
            'subject',
            'created_at',
        ];

        if($extension == 'pdf'){
            $data = [
                'ticket_data' => $exportableData->cursor(),
                'title'       => translate('Download Pdf')
            ];

            $pdf = PDF::loadView($view, $data);
            return $pdf->download('ticket.pdf');

        }
        
        elseif($extension == 'csv'){

 
            $tempFile = tempnam(sys_get_temp_dir(), 'ticket_');
          
            $handle   = fopen($tempFile, 'w');

            foreach ([$headers] as $contact) {
                fputcsv($handle, $contact);
            }

            $exportableData->select($headers)
                ->chunk(200, function ($contact) use ($handle) {
                    foreach ($contact->toArray() as $contact) {
                        fputcsv($handle, $contact);
                    }
                });

            fclose($handle);

            $header = [
                'Content-Description'   => 'File Transfer',
                'Content-Type'          => 'application/octet-stream',
                'Content-Disposition'   => 'attachment; filename=ticket.csv',
            ];

            $response                   = response()->download($tempFile, 'ticket.csv', $header);


            register_shutdown_function(function () use ($tempFile) {
                if (file_exists($tempFile)) {
                    unlink($tempFile);
                }
            });

            return $response;



        }

     }

     /**
      * notify user 
      */

      public function notifyUser($user,$message,$subkey,$template,$mailCode ,$route){

           $notificatios_settings = $user->notification_settings ? json_decode($user->notification_settings,true) : [];
            if(site_settings("database_notifications") == (StatusEnum::true)->status()){
                $data = [
                    'route'    => $route,
                    'messsage' => $message
                ];
                $notification = SendNotification::database_notifications($data,$user,auth_user()->id,NotifyStatus::USER);

                /** trigger pusher with data if browser notification is on for agent */
                if(isset($notificatios_settings['browser'][$subkey]) && $notificatios_settings['browser'][$subkey] == (StatusEnum::true)->status()){
                    $push_data ['notifications_data'] = [
                        'counter'          => false,
                        'data'             => json_decode($notification->data,true),
                        'notification_for' => $notification->notification_for,
                        'notify_id'        => $notification->notify_id,
                        'for'              => $subkey,
                    ] ; 
                    try {
                        $this->triggerPusher($push_data);
                    } catch (\Exception $e) {
                        
                    }
                }
            }

            if(site_settings("email_notifications") == (StatusEnum::true)->status()  && isset( $notificatios_settings['email'][$subkey]) && $notificatios_settings['email'][$subkey] == (StatusEnum::true)->status()){
                SendEmailJob::dispatch($user,$template,$mailCode);
            }

            if(site_settings("sms_notifications") == (StatusEnum::true)->status() && isset( $notificatios_settings['sms'][$subkey]) &&  $notificatios_settings['sms'][$subkey] == (StatusEnum::true)->status()){
                SendSmsJob::dispatch($user,$template,$mailCode);
            }
      }


    public function bulckAction($request){


        if($request->status){

            $message = translate('Status Updated Successfully');

            SupportTicket::whereIn('id',$request->ticket_id)->update([
                   'status'=> $request->status
            ]);

            $tickets =  SupportTicket::whereIn('id',$request->ticket_id)->get();


            foreach($tickets->chunk(paginateNumber())  as $chunksTickets){

                foreach($chunksTickets as $ticket){

                    if($request->status == TicketStatus::SOLVED->value){

                        if ((auth_user()->agent == StatusEnum::true->status()) && ($ticket->solved_request == SupportTicket::REJECTED || !$ticket->solved_request)) {
                            $ticket->solved_request    = SupportTicket::REQUESTED;
                            $ticket->solved_request_at = Carbon::now();
                            $ticket->requested_by      = auth_user()->id;
                            $this->notifyAdmin($ticket);
                            
                        } else {
                            $ticket->solved_at = $ticket->solved_at ? $ticket->solved_at : Carbon::now() ;
                        }
                    }

                    $ticket->status =  $request->status;

                    $ticket->save();
                }

            }


        }
        elseif($request->assign){

            $this->assignTicket($request);
            $message = translate('Ticket Assigned Successfully');
        }
        else{

            $this->deleteTicket($request->ticket_id);
            $message = translate('Ticket Deleted Successfully');
        }


        return $message;


    }



    /**assign tickets to admin/agent */
    public function assignTicket($request){


        $admin   = Admin::with(['tickets'])
        ->where('id',$request->assign)
        ->first();

        $tickets = SupportTicket::whereIn("id" ,$request->ticket_id )->get();

        $attchableData = [];
        
        $sortNotes = "Ticket Assigned BY SuperAdmin";
        $message   = translate("You Have a New Assigned Ticket By").auth_user()->name;
        $assignBy  = auth_user()->id;

        AgentTicket::whereIn('ticket_id',$request->ticket_id)->delete();
    
        $subKey = "admin_assign_ticket";

        if($admin->id == auth_user()->id){ 
            $sortNotes = "Ticket Assigned By Me";
            $assignBy  = null;
        }
        if(auth_user()->agent == StatusEnum::true->status()){
            $subKey    = "agent_assign_ticket";
            $sortNotes = "Ticket Assigned By Agent";
        }

        foreach($tickets  as $uniqueticket){

            if($request->priority_id ){
                $uniqueticket->priority_id = $request->priority_id;
                $uniqueticket->save();
            }

            $route = route("admin.ticket.view",$uniqueticket->ticket_number);
            
            $data  =   
            [ 
                "agent_id"    => $admin->id , 
                "ticket_id"   => $uniqueticket->id,
                "short_notes" => $request->short_note ? $request->short_note : $sortNotes ,
                "assigned_by" => $assignBy ,
            ];
            array_push($attchableData,$data);
        
            if($admin->id != auth_user()->id){ 
                $mailCode = [
                    "role"           => auth_user()->agent == StatusEnum::true->status() ? 'Agent' : "SuperAdmin",
                    "name"           => auth_user()->name,
                    "ticket_number"  => $uniqueticket->ticket_number,
                    "link"           => ($route)
                ];
                $this->notifyAgent(null,$admin, $message ,$subKey ,$route ,"SUPPORT_TICKET_ASSIGN",
                $mailCode);
            }
            
            if(auth_user()->agent ==  StatusEnum::true->status()){
                
                $message = "Hello Superadmin ! Agent " . auth_user()->name. " just Assign A ticket To Agent " .$admin->name;
                $mailCode = [
                    "name"           => auth_user()->name,
                    "assigned_to"    => $admin->name,
                    "ticket_number"  => $uniqueticket->ticket_number,
                    "link"           => ($route)
                ];

                $this->notifyAgent($uniqueticket,null, $message ,$subKey ,$route ,"SUPPORT_TICKET_ASSIGN_BY_AGENT",
                $mailCode);
            }


        }
    
        $admin->tickets()->attach($attchableData);

    }



    /**
     * delete tickets 
     */
    public function deleteTicket($ticketIds){

        foreach($ticketIds as $ticket_id){
            
            $ticket = SupportTicket::with(["messages",'agents'])->withCount('messages')->where('id',$ticket_id)->first();
            if($ticket){
                if($ticket->messages_count > 0){
                    foreach($ticket->messages as $message){
                        $ticketFiles = json_decode($message->file,true);
                        $this->unlinkFile($ticketFiles ? $ticketFiles :[]  , getFilePaths()['ticket']['path']); 
                        $message->delete();
                    }
                }
                $ticket->agents()->detach();
                $ticket->delete();
            }
        }
    }



    
    /** unlink ticket files */
    public function unlinkFile($files ,$location){
        
        foreach($files as $file){
            remove_file($location ,$file );
        }
    }




     /** proceess ticket data */
     public function processData($tickets,$request) :array {


        $ticketData     = $tickets;
        $status         = true;
        $ticket_status  = [];
        $categories     = Category::agent( $request->agent_id)
        ->withCount(['tickets as tickets_count' => function($q) use($request){
            if(auth_user()->agent == StatusEnum::true->status() || $request->agent_id){
                $q->agent( $request->agent_id);
            }
        } ])
        ->having("tickets_count" ,">", 0)
        ->where("ticket_display_flag",StatusEnum::true->status())->latest()
        ->get();

        $category_active = $request->category_id ? $request->category_id : null;
        $status_active   = $request->status ? $request->status : null;
        $tickets         = $tickets->filter($request)
        ->paginate(paginateNumber(), ['*'], 'page', $request->page) ? $tickets->filter($request)
        ->paginate(paginateNumber(), ['*'], 'page', $request->page) : [] ;

        $tag             = $request->tag;
        $nextPageNumber  = $tickets->currentPage() + 1;
        $status          = count($tickets)  == 0 ? false :true;
        $currentPage     = $tickets->perpage() * $tickets->currentpage() > $tickets->total()  ? $tickets->total()  :  $tickets->perpage() * $tickets->currentpage();

        $ticketStatus    = TicketStatus::toArray();

        $tickets_counter =  SupportTicket::agent($request->agent_id)
        ->filter($request,false)->groupBy('status')
        ->select('status', DB::raw('count(*) as count'))
        ->pluck('count', 'status')
        ->toArray();



        return ([

            "tickets"             => $tickets,
            'total_ticket'        => $tickets->total(),
            'current_page_ticket' => $currentPage,
            'status'              => $status,
            "ticket_html"         => view('admin.ticket.list', compact('tickets'))->render(),
            "categories_html"     => view('admin.ticket.categories', compact('categories','tickets','category_active'))->render(),
            "status_html"         => view('admin.ticket.status', compact('ticketStatus',"tickets_counter",'status_active'))->render(),
            "tag_html"            => view('admin.ticket.tags', compact('ticketData','tag'))->render(),
        ]);
    }

    





}