<?php
namespace App\Http\Utility;

use App\Enums\StatusEnum;
use App\Models\MailConfiguration;
use Illuminate\Support\Facades\Mail;

use App\Models\EmailTemplates;

class SendMail
{
    
    
    public static function MailNotification($userInfo, $emailTemplate, $code = [])
    {
      
    	$mailConfiguration = MailConfiguration::where('id', site_settings('email_gateway_id'))->where('status', (StatusEnum::true)->status())->first();
    	if(!$mailConfiguration){
    		return;
    	}
       
        $emailTemplate = EmailTemplates::where('slug', $emailTemplate)->first();

        $messages = str_replace("{{username}}", @$userInfo->username ? $userInfo->username : $userInfo->name , site_settings('default_mail_template'));
        $messages = str_replace("{{message}}", @$emailTemplate->body, $messages);
        $messages = str_replace("{{site_name}}", @site_settings('site_name'), $messages);
        $messages = str_replace("{{logo}}", @getImageUrl(getFilePaths()['site_logo']['path']."/".site_settings('site_logo_lg')), $messages);
        foreach ($code as $key => $value) {
            $messages = str_replace('{{' . $key . '}}', $value, $messages);
        }

    	if($mailConfiguration->name === "PHP MAIL"){
    		self::SendPHPmail(site_settings('email'),  site_settings('site_name'), $userInfo->email, $emailTemplate->subject, $messages);
    	}
        elseif($mailConfiguration->name === "SMTP"){
    		self::SendSMTPMail($mailConfiguration->driver_information->from->address, $userInfo->email,  site_settings('site_name'), $emailTemplate->subject, $messages);
    	}
        elseif($mailConfiguration->name === "SendGrid Api"){
            self::SendGrid($mailConfiguration->driver_information->from->address,  site_settings('site_name'), $userInfo->email, $emailTemplate->subject, $messages, @$mailConfiguration->driver_information->app_key);
        }
    }

    public static function SendPHPmail($emailFrom, $sitename, $emailTo, $subject, $messages)
    {
        $headers = "From: $sitename <$emailFrom> \r\n";
        $headers .= "Reply-To: $sitename <$emailFrom> \r\n";
        $headers .= "MIME-Version: 1.0\r\n";
        $headers .= "Content-Type: text/html; charset=utf-8\r\n";
        try {
            @mail($emailTo, $subject, $messages, $headers); 
            return true;
        } catch (\Exception $e) {
            return false;
        }
    }

    public static function SendSMTPMail($emailFrom, $emailTo, $fromName, $subject, $messages)
    {  

        try{ 
            Mail::send([], [], function ($message) use ($messages, $emailFrom, $fromName, $emailTo, $subject) {
                        $message->to($emailTo) 
                    ->subject($subject)
                    ->from($emailFrom,$fromName)
                    ->setBody($messages, 'text/html','utf-8');
            });
            return true;

        }catch (\Exception $e){  
            return false;
        } 
    } 

    public static function SendGrid($emailFrom,$fromName, $emailTo, $subject, $messages, $credentials)
    { 
        try{
            $email = new \SendGrid\Mail\Mail();
            $email->setFrom($emailFrom, $fromName);
            $email->setSubject($subject);
            $email->addTo($emailTo);
            $email->addContent("text/html", $messages);
            $sendgrid = new \SendGrid($credentials);
            $response = $sendgrid->send($email);
            return true;
        }catch(\Exception $e){
            return false;
        }
    }
}